! $Id: identifying.prg,v 1.5 2002/01/30 09:11:50 cguirao Exp $
! $Name: FIASCO-1_3 $
!
! AUTHOR: C.Guirao 
!
! NAME: indentifying.prg
!
! INPUT: FITS or BDF 1-dim spectrum with absortion lines to be identified.
!
!!	It uses an MIDAS table containing two columns (default solar.tbl)
!	WAVE & ION, from 4500 to 7000 angstroms. This table is a subset 
!	of ills.dat (what and from where) created by the author with
!	the strongest lines identified by NRAO in the solar spectrum.
!
!	The 'solar.tbl' can be created with the procedure 'solar.prg'
!	and the input ASCII file 'solar.dat'
!
!	The procedure, first, creates a table with the identified 
!	absorption lines from the input file that conform the 
!	threshold and with requirements (parameters p2 and p3).
!
!	This table is filtered with a given wavelength (p4) and having 
!	a peak value less than (p2). 
!
!	The filtered table is confronted with table solar.tbl and those
!	lines identified with an ION element within tolerance (p6) are
!       then overploted.
!
! OUTPUT: Plots the original spectrum with labels for identifed lines.
!
! USAGE: @@ identifying file [table] [thres] [width] [peak] [wave_range] [tol]
!
! EXAMPLES:
! 
!        @@ identifying.prg sun.fit ? ? ? ? ? 
!        @@ identifying.prg sirius.bdf ? ? .95 5000,5500 .5
!
!*******************************************************************************
! ECHO/FULL
!
! INPUT PARAMS:
! p1 REQUIRED: FITS or BDF image of 1dim spectrum already wavelength calibrated
! p2 OPTIONAL: PEAK threshold. Value below/above (ABSORPTION/EMISSION) to be labeled 
! p3 OPTIONAL: Threshold. Value above noise to be identified as a line
! p4 OPTIONAL: Width of the local histogram window
! p5 OPTIONAL: Wavelength range to be labeled 
! p6 OPTIONAL: Tolerance for matching
!
! Edit here to change defaults:
!
 DEFINE/PARA p1 ? I "Enter FITS/BDF image with calibrate spectrum to be identified? "
 DEFINE/PARA p2 + 	NUM
 DEFINE/PARA p3 +	NUM
 DEFINE/PARA p4 +	NUM
 DEFINE/PARA p5 +	NUM
 DEFINE/PARA p6 .4	NUM
!
! Convert paramaters into variables:
!
 DEFINE/LOCA cmd/c/1/20         "identifying"
 DEFINE/LOCA input/c/1/40	{p1}
 DEFINE/LOCA frame/c/1/40
 DEFINE/LOCA mode/c/1/40	"ABSORPTION"
 DEFINE/LOCA line_table/c/1/40  "solar.tbl"
 DEFINE/LOCA thres/r/1/1
 DEFINE/LOCA width/r/1/1
 DEFINE/LOCA max/r/1/1         
 DEFINE/LOCA min/r/1/1         
 DEFINE/LOCA peak/r/1/1         
 DEFINE/LOCA tolerance/r/1/1    {p6}
 DEFINE/LOCA wave_central/i/1/1 0
 DEFINE/LOCA basename/c/1/40    "" ? +lower_levels
 DEFINE/LOCA ext/c/1/10         "" ? +lower_levels
 DEFINE/LOCA size_ccd/i/1/2
 DEFINE/LOCA naxis/i/1/1
 DEFINE/LOCA loop/i/1/1		0
 DEFINE/LOCA row/i/1/1		0
!
! Gets basename and extension of line_table file
!
IF m$exist(line_table) .eq. 0 THEN
  WRITE/OUT "{cmd}: Executing solar.prg to generate {line_table}"
  @@ solar.prg
ENDIF
  
!
! Gets basename and extension of input file
!
 @@ file_ext {input}
 frame = basename
!
! Convert FITS format into MIDAS BDF format
!
 IF ext .ne. ".bdf" THEN
   WRITE/OUT "{cmd}: converting {input} into {frame}"
   @@ compute_fits {input} {frame}
 ENDIF
!
! Get the size in pixels of the CCD
!
 size_ccd(1) = m$value({frame},npix(1))
 size_ccd(2) = m$value({frame},npix(2))
 naxis       = m$value({frame},naxis)
!
! This procedure one works with 1dim spectra
!
 IF naxis .ne. 1 THEN
   WRITE/OUT "{cmd}: Only with 1-dim spectra"
   WRITE/OUT "Usage: @@ {cmd} file [table] [thres] [width] [peak] [wave_range] [tol]"
   RETURN/EXIT
 ENDIF
!
 SHOW/CONTEXT long >NULL
 IF OUTPUTI(5) .eq. 1 THEN
   CLEAN/LONG
 ENDIF
 SET/CONTEXT long >NULL
! RESET/DISP >NULL
!
! Initial threshold is standard_deviation for {frame}/10
! Width is 4 * STEP for {frame}
!
 IF p3 .eq. "+" THEN
   STATIS/IMA {frame} >NULL
   thres = OUTPUTR(4) / 10
!   WRITE/OUT "{cmd}: median value in {frame} = {thres}"
 ELSE
   thres = {p3}
 ENDIF
 IF p4 .eq. "+" THEN
   width = m$value({frame},step)
   width = 40 * width
!   WRITE/OUT "{cmd}: width value in {frame} = {width}"
 ELSE
   width = {p4}
 ENDIF
!
 SET/GRAP DEFAULT=
 SET/GRAP pm=1 font=1 xa=auto ya=auto
! DELETE/GRAP 0 >NULL
 GRAPH/SPEC ? ? 0
 PLOT/ROW {frame} >NULL
!
! Values below PEAK to be searched in table
!
 IF p2 .eq. "+" THEN
   STATIS/IMA {frame} >NULL
   max = OUTPUTR(2)
   min = OUTPUTR(1)
   peak = max - ((max - min)/4)
!   WRITE/OUT "{cmd}: threshold for PEAK in {frame} = {peak}"
 ELSE
   peak = {p2}
 ENDIF
!
 WRITE/OUT
 WRITE/OUT "FRAME        = {frame}"
 WRITE/OUT "LINE_CATALOG = {line_table}"
 WRITE/OUT "PEAK         = {peak}"
 WRITE/OUT "THRESHOLD    = {thres}"
 WRITE/OUT "WIDTH        = {width}"
! WRITE/OUT "MODE         = {mode}"
 IF p5 .ne. "+" THEN
   DEFINE/LOCA range/r/1/2 {p5}
   WRITE/OUT "RANGE        = {range(1)},{range(2)}"
 ELSE
   WRITE/OUT "RANGE        = FULL RANGE"
 ENDIF
 WRITE/OUT "TOLERANCE    = {tolerance}"
 WRITE/OUT
!
 askContinue:
 WRITE/KEYW reply/c/1/1 y
 WRITE/OUT "Searching lines with above values"
 INQUIRE/KEYW reply "Continue [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue
 ELSEIF reply(:1) .eq. "n" THEN
   WRITE/OUT "Usage: @@ {cmd} file [peak] [thres] [width] [wave_range] [tol]"
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askContinue
 ENDIF
 continue:

!
 ECHO/FULL
 SEARCH/LONG {frame} thres={thres} width={width} ? ? GRAVITY {mode}
 ECHO/OFF
 PLOT/SEARCH >NULL
 SHOW/TABL line.tbl >NULL
 row = OUTPUTI(2)
!
 WRITE/OUT
 WRITE/OUT "DETECTIONS   = {row}"
 WRITE/OUT
!
 askContinue1:
 WRITE/KEYW reply/c/1/1 y
 WRITE/OUT "Matching detected lines in {line_table} with above values"
 INQUIRE/KEYW reply "Continue [yn] (y)?"
 IF reply(:1) .eq. "y" THEN
   GOTO continue1
 ELSEIF reply(:1) .eq. "n" THEN
   WRITE/OUT "Usage: @@ {cmd} file [peak] [thres] [width] [wave_range] [tol]"
   ECHO/OFF
   RETURN/EXIT
 ELSE
   GOTO askContinue1
 ENDIF
 continue1:
!
! You may want to reduce further the table of identified lines
!
! Select those entries with peak less than 0.9
! or those between a given wavelength range.
!
 SELECT/TABLE line.tbl :PEAK .LT. {peak}
 COPY/TABL line.tbl nline.tbl
 RENAME/TABL nline.tbl line.tbl
 SHOW/TABL line.tbl >NULL
 row = OUTPUTI(2)
 WRITE/OUT "{cmd}: {row} lines detected with :PEAK LessThan {peak}"
 IF p5 .ne. "+" THEN
   SELECT/TABLE line.tbl :X .GT.  {range(1)} .AND. :X .LT. {range(2)}
   COPY/TABL line.tbl nline.tbl
   RENAME/TABL nline.tbl line.tbl
   SHOW/TABL line.tbl >NULL
   row = OUTPUTI(2)
   WRITE/OUT "{cmd}: {row} lines within range {range(1)},{range(2)}"
 ENDIF
!
! Or sort the table by peak (intensity) 
! SORT/TABLE line.tbl :PEAK
!
!
 SET/GRAP DEFAULT=
 SET/GRAP pm=1 font=1 xa=auto ya=auto
! DELETE/GRAP 0 >NULL
 GRAPH/SPEC ? ? 0
 PLOT/SEARCH >NULL
!
!
 JOIN/TABLE line.tbl :X {line_table} :WAVE result.tbl {tolerance}
 SHOW/TABL result.tbl >NULL
 row = OUTPUTI(2)
 WRITE/OUT "{cmd}: {row} lines detected  within tolerance {tolerance}"
!
! GRAPH/SPEC ? ? 1
! SET/GRAP DEFAULT=
 SET/GRAP pm=1 font=1 xa=auto ya=auto
! DELETE/GRAP 1 >NULL
! PLOT/ROW {frame} >NULL
!
 SET/GRAP COLOUR=4
 IF mode .eq. "ABSORPTION" THEN
   OVERPLOT/IDENT result.tbl :X_1 :ION_2 BOTTOM >NULL
 ELSEIF mode .eq. "EMISSION" THEN
   OVERPLOT/IDENT result.tbl :X_1 :ION_2 TOP >NULL
 ENDIF
 SET/GRAP COLOUR=1
!
! Clean all temporary files
!
 $rm -f NULL
!
 ECHO/OFF 
